#!BPY
""" Registration info for Blender menus:
Name: 'BZW Toolkit v2.0.4-beta2'
Blender: 242
Group: 'Object'
Tooltip: 'Add and edit BZflag style objects within blender.'
"""
__author__ = ("'Teppic'")
__version__ = "2.0.4-beta2"
__bpydoc__ = """\
This script helps exports to BZW 2.0.4 format.
Usage:
Run this script from "Object/Bzwtoolkit" menu.
"""
# 
# Copyright (C) 2006: A.D.Foster
#
# ***** BEGIN GPL LICENSE BLOCK *****
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# ***** END GPL LICENCE BLOCK *****

import Blender
import sys
from Blender import Draw, BGL, Object
from bzflag import BZtypeList
from bzflag import BZsceneWriter
from bzflag import BZRegistry

class BZWTOOLKIT:

	def __init__(self):
		# ####################################
		# New Section to kill off BZ-config.py
		# ####################################
		if 'linux' not in sys.platform:
			try:
				BZPrefs=Blender.Registry.GetKey('BZFlag', True)
				bztexpath=BZPrefs['bztexpath']
				cstexpath=BZPrefs['cstexpath']
				cstexhost=BZPrefs['cstexhost']
				bzcachepath=BZPrefs['bzcachepath']
				ro=BZPrefs['ro']
				print 'BZFlag settings found, setting to user preferences'
			except:
				BZPrefs={'bztexpath':'/','cstexpath':'/','cstexhost':'/','bzcachepath':'/','ro':4,}
				Blender.Registry.SetKey('BZFlag', BZPrefs, True)
				print 'No BZFlag settings found, re-setting to defaults'
		else:
			try:
				BZRegistry.LoadConfigData('BZFlag')
				BZPrefs=Blender.Registry.GetKey('BZFlag')
				bztexpath=BZPrefs['bztexpath']
				cstexpath=BZPrefs['cstexpath']
				cstexhost=BZPrefs['cstexhost']
				bzcachepath=BZPrefs['bzcachepath']
				ro=BZPrefs['ro']
				print 'BZFlag settings found, setting to user preferences'
				Blender.Registry.SetKey('BZFlag', BZPrefs)
			except:
				BZPrefs={'bztexpath':'/','cstexpath':'/','cstexhost':'/','bzcachepath':'/','ro':4,}
				Blender.Registry.SetKey('BZFlag', BZPrefs)
				print 'No BZFlag settings found, re-setting to defaults'	
				
				
		# ####################################
		# New Section to kill off BZ-config.py
		# ####################################
		
		self.objectlist={} 	 		#list of all objects withou bzflag variables
		self.BZobjectlist={} 		#list of objects with bzflag variables set
		self.BZmatobjectlist={}		#list of 'bzflag' defined materials
		self.BZmatIndexList={}		#backwardly readble above instance, yeah I could retrive it but can't be arsed
		self.telelist = {}   		#List of tele objects referencable by index number
		self.menulist=''		    #As above but for Draw.Menu function calls
		self.BZmenulist=''			#As above but for Draw.Menu function calls
		self.BAmatmenuList=''		#As above but for Draw.Menu function calls
		self.indexnumber=1			#for correlation between selectedobject.val
		self.BZindexnumber=1		#for correlation between selectedBZobject.val
		self.BZmatindexnumber=1		#for correlation between selectedBZmatobject.val
		self.selectedObject='1'	 	#set the default index, will become Draw.Menu class object
		self.selectedBZObject='1'	#set the default index, will become Draw.Menu class object
		self.selectedBZmatObject='1'#set the default index, will become Draw.Menu class object
		self.typeCache={}			#holds number versus type for dynamic gui objects
		self.message="Hello...."	#can't remember.......
		self.materialval={}			#holds default values, then menu values
		self.material={}			#holds menu objects
		self.stringval={}			#holds default values, then string values
		self.string={}				#holds string objects
		self.attribval={}			#holds default values, then string values
		self.attrib={}				#holds string objects
		self.bool={}				#holds toggle objects FIXME, should be done via event switching
		self.boolval={}				#holds default values, then bool states
		self.AddObjectList=(	'world','waterLevel','dynamicColor','textureMatrix','material','physics',
							'define','group','meshbox','meshpyr','arc','cone','sphere','box','pyramid',
							'link','teleporter','base','weapon','zone')
		self.ExceptList=('size','position','rotation','name')					

#########
#main gui
#########
	def event(self, evt, val):
		if evt==Draw.ESCKEY and val==1:
			Draw.Exit()
			return
		elif val != 0:        
			return
		else: 
			return
			
	def button_event(self,evt):
		if evt == 1:
			Draw.Exit()
			return
		elif evt == 2:
			self.addNew()
			return
		elif evt == 4:
			self.editExisting()
			return
		elif evt==5:
			obj=Blender.Object.Get(self.objectlist[int(self.selectedObject.val)])
			self.template=BZtypeList.mesh()
			self.ExceptList=('spin','shear','scale','shift','name','position','size','rotation')
			for t in self.template.bools:
				if t not in self.ExceptList:self.propertyCheck(obj,t,getattr(self.template,t,False))
			for t in self.template.strings:
				if t not in self.ExceptList:self.propertyCheck(obj,t,getattr(self.template,t,''))
			for t in self.template.attribs:
				if t not in self.ExceptList:self.propertyCheck(obj,t,getattr(self.template,t,''))
			self.propertyCheck(obj,'bzflag','mesh')
			self.propertyCheck(obj,'name',obj.name)
			self.getobjects(Blender.Scene.GetCurrent())	
		elif evt==6:
			self.editPrefs()
		elif evt==7:
			#self.faceCleaner()
			objs=Blender.Object.GetSelected()
			for obj in objs:
				mesh=obj.getData()
				mesh.hasFaceUV(1)
				for face in mesh.faces:
##					face.flag=1
##					face.mode=5
					face.transp=0
				mesh.update()
				print "%s faces are clean." % obj.name
		elif evt==99:
			self.getobjects(Blender.Scene.GetCurrent())	
		return

	def gui(self):
		wid,hig=Blender.Window.GetAreaSize()
		a=wid/3; x=hig/3
		if x>30: x=30
		if a>150: a=150
		b=a/20;	c=a-(2*b) ;	y=x/10;	z=x-(2*y)
		BGL.glClear(BGL.GL_COLOR_BUFFER_BIT)
		BGL.glColor3f(1,1,1)
		#bottom row
		Draw.PushButton("<< Exit", 1, b, y, c, z,"Exit the toolkit")
		Draw.PushButton("Edit Settings", 6, a+b,y,c,z,"Edit your settings used for the importer and exporter") 
		Draw.PushButton("Update", 99, 2*a+b, y, c, z,"Update the object lists from the scene")
		#mid row
		Draw.PushButton("Face Cleaner", 7, b,x+y,c,z,"Clear unwanted shootthrough etc attributes on meah faces.(workaround until gui is done)")
		self.selectedBZObject=Draw.Menu(self.BZmenulist, 3, a+b, x+y, c, z, self.BZindexnumber ,"Select the Object to change")
		self.selectedObject=Draw.Menu(self.menulist,99,2*a+b,x+y,c,z,self.indexnumber,'Select the mesh to set the export flag on')
		#top row
		Draw.PushButton("Add Object", 2, b, 2*x+y, c, z,"Add a new BZflag object to the scene")
		Draw.PushButton("Edit Object" , 4,a+b, 2*x+y, c, z,"Edit BZflag object selected below")
		Draw.PushButton("Mesh Export", 5, 2*a+b, 2*x+y, c, z,"Set the export flag on the mesh listed below")
		BGL.glRasterPos2i(int(b),int(1.5*x+y))
		Draw.Text(self.message)
		
######################################################
#add gui
######################################################
	def addgui(self):
		wid,hig=Blender.Window.GetAreaSize()
		a=wid/5; x=hig/6
		if x>30: x=30
		if a>150: a=150
		b=a/20;	c=a-(2*b) ;	y=x/10;	z=x-(2*y)
		BGL.glClearColor(0.5,0.5,0.5,1)	
		BGL.glClear(BGL.GL_COLOR_BUFFER_BIT)
		BGL.glColor3f(1,1,1)
		BGL.glRasterPos2i(b, 5*x+y)
		Draw.Text("Objects:")
		BGL.glRasterPos2i(2*a+b, 5.5*x+y)		
		Draw.Text("Select the item you would like to add")			
		for i in range(0,len(self.AddObjectList)):
			Draw.PushButton(self.AddObjectList[i],i+1,(i-((i/5)*5))*a+b, ((i/5)+1)*x+y,c,z)
		Draw.PushButton("<< Back", 100, b, y, c, z,"Exit to the toolkit")

	def addevent(self,evt,val):
		if val != 0:
			if evt==Draw.ESCKEY:
				self.getobjects(Blender.Scene.GetCurrent())	
	def addbutton_event(self,evt):
		if evt==0:
			pass
		elif evt==100:
			self.getobjects(Blender.Scene.GetCurrent())
		else:
			obj=BZtypeList.Objects[self.AddObjectList[evt-1]]()
			obj.name=self.checkName(obj.name)
			try:
				mesh=Blender.Mesh.Get(obj.name)
				mesh.name='defucnt'
				print mesh.name
			except:
				pass
			module=BZsceneWriter.Object[obj.type()]
			writer=module.importer(obj)
			writer.run()		
			
######################################################
#edit gui
######################################################
	def editgui(self):
		wid,hig=Blender.Window.GetAreaSize()
		a=wid/5; x=hig/6
		if x>30: x=30
		if a>150: a=150
		b=a/20;	c=a-(2*b) ;	y=x/10;	z=x-(2*y)
		BGL.glClearColor(0.5,0.5,0.5,1)	
		BGL.glClear(BGL.GL_COLOR_BUFFER_BIT)
		BGL.glColor3f(1,1,1)
		BGL.glRasterPos2i(b, 5*a+b)
		Draw.Text("Editing options for: %s" % self.template.name)		
		i=0
		for tog in self.template.bools:
			if tog not in self.ExceptList:
				Draw.Toggle(tog,i+1,(i-((i/5)*5))*a+b, ((i/5)+1)*x+y,c,z,self.boolval[tog])
				self.typeCache[i+1]=tog
				i+=1
		for st in self.template.strings:
			if st not in self.ExceptList:
				if st=='matref':
					if not self.stringval[st]:
						self.stringval[st]=1
					elif type(self.stringval[st])=='str':#convert to a number to reference against the menu list
						self.stringval[st]=[self.BZmatIndexList[self.stringval[st]]]
					self.string[st]=Draw.Menu('matref%s' % self.BZmatmenulist,i+1,(i-((i/5)*5))*a+b, ((i/5)+1)*x+y,c,z, self.stringval[st] ,"Select the objects material")
					self.typeCache[i+1]=st
					i+=1
				else:
					self.string[st]=Draw.String(st+': ',i+1,(i-((i/5)*5))*a+b, ((i/5)+1)*x+y,c,z, self.stringval[st],100)
					self.typeCache[a+1]=st
					i+=1
		for mat in self.template.materials:
			self.material[mat]=Draw.Menu('%s%s'	% (mat,self.BZmatmenulist), i+1,(i-((i/5)*5))*a+b, ((i/5)+1)*x+y,c,z, self.materialval[mat] ,"Select faces' material")
			self.typeCache[i+1]=mat
			i+=1
		for st in self.template.attribs:
			if st not in self.ExceptList:
				self.attrib[st]=Draw.String(st+': ',i+1,(i-((i/5)*5))*a+b, ((i/5)+1)*x+y,c,z, self.attribval[st],100)
				self.typeCache[i+1]=st
				i+=1									
		Draw.PushButton("Cancel", 100, b, y, c, z,"Exit to the toolkit")
		Draw.PushButton("Apply", 99, 4*a+b, y, c, z,"Apply the changes")

	def editevent(self,evt,val):
		if val != 0:
			if evt==Draw.ESCKEY:
				self.getobjects(Blender.Scene.GetCurrent())	
	def editbutton_event(self,evt):
		try:
			data=self.typeCache[evt]
		except KeyError:
			data=None
		if evt==0:
			return
		elif evt==100:
			self.getobjects(Blender.Scene.GetCurrent())
		elif evt==99:
			self.applyExisting()
		elif data:
			if data in self.template.materials:
				indval=self.material[data].val
				self.materialval[data]=int(indval)
			elif data =='matref':
				indval=self.string[data].val
				self.stringval[data]=int(indval)
			elif data in self.template.strings:
				self.stringval[data]=self.string[data].val
			elif data in self.template.attribs:
				self.attribval[data]=self.attrib[data].val
			elif data in self.template.bools:
				self.boolval[data]=1-self.boolval[data]
		Draw.Redraw(1)
		return

######################################################
#preferences gui
######################################################
	def prefevent(self, evt, val):
		if val != 0:
			if evt==Draw.ESCKEY:
				self.getobjects(Blender.Scene.GetCurrent())
		return
			
	def prefbutton_event(self,evt):
		if evt == 1:
			self.getobjects(Blender.Scene.GetCurrent())	
			return
		elif evt == 2:
			self.prefGet()
			return
		elif evt == 3:
			self.prefSave()
			return
		elif evt == 4:
			self.bztexpath=self.nbztexpath.val
			return
		elif evt == 5:
			self.cstexpath=self.ncstexpath.val
			return
		elif evt == 6:
			self.bzcachepath=self.nbzcachepath.val
			return
		elif evt == 7:
			self.cstexhost=self.ncstexhost.val
			return
		elif evt == 8:
			self.ro=self.nro.val
			return
		elif evt == 9:
			Blender.Window.FileSelector(self.callback9,"Select Path",self.bztexpath)
			return
		elif evt == 10:
			Blender.Window.FileSelector(self.callback10,"Select Path",self.cstexpath)
			return
		elif evt == 11:
			Blender.Window.FileSelector(self.callback11,"Select Path",self.bzcachepath)
			return
		return

	def prefgui(self):
		wid,hig=Blender.Window.GetAreaSize()
		a=int(wid/3); x=int(hig/6)
		if x>30: x=30
		if a>180: a=180
		b=a/20;	c=a-(2*b) ;	y=x/10;	z=x-(2*y)
		#BGL.glClear(BGL.GL_COLOR_BUFFER_BIT)
		#BGL.glColor3f(1,1,1)
		#bottom row
		Draw.PushButton("<< Back",	 1, b, y, c, z,"Back to the main menu")
		Draw.PushButton("Reset",	 2, a+b,y,c,z,"Reset the values to the last saved") 
		Draw.PushButton("Apply",	 3, 2*a+b, y, c, z,"Apply the settings to the blender registry")
		#other rows
		Draw.PushButton(""										, 9,  c*3+b*2, 4*x+y, b*3, z,"Browse for directory in file selector")
		Draw.PushButton(""										, 10, c*3+b*2, 3*x+y, b*3, z,"Browse for directory in file selector")
		Draw.PushButton(""										, 11, c*3+b*2, 2*x+y, b*3, z,"Browse for directory in file selector")
		#Draw.PushButton(""										, 12, c*3+b*2, 1*x+y, b*3, z,"Browse for directory in file selector")
		
		self.nbztexpath=Draw.String('BZFlag Texture Path: '		, 4, b, 4*x+y, c*3+b, z, self.bztexpath, 100, 'The directory on your machine where BZFlag stores standard textures')
		self.ncstexpath=Draw.String('Custom Texture Path: '		, 5, b, 3*x+y, c*3+b, z, self.cstexpath, 100, 'The directory on your machine where you store custom textures')
		self.nbzcachepath=Draw.String('Cached Texture Path: '	, 6, b, 2*x+y, c*3+b, z, self.bzcachepath, 100, 'The directory on your machine where BZFlag stores custom textures')
		self.ncstexhost=Draw.String('Custom Texture Host: '		, 7, b, 1*x+y, c*3+b*4, z, self.cstexhost, 100, 'The host you use to serve custom textures to other users')
		self.nro=Draw.Slider('Rounding Accuracy: '				, 8, b, 5*x+y, c*3+b*4, z, self.ro , 0, 6, 1, "The number of decimal places the exporter will use when writing out files")
		#done
	
######################################################
#face cleaner gui
######################################################
	def cleanevent(self, evt, val):
		if val != 0:
			if evt==Draw.ESCKEY:
				self.getobjects(Blender.Scene.GetCurrent())
		return
			
	def cleanbutton_event(self,evt):
		if evt == 1:
			pass
		#elif evt == 2:
##
##		elif evt == 3:
##
##		elif evt == 4:
##
##		elif evt == 5:
##
##		elif evt == 6:
##
##		elif evt == 7:
##
##		elif evt == 8:
##
##		elif evt == 9:
##
##		elif evt == 10:
##
##		elif evt == 11:

		return

	def cleangui(self):
		wid,hig=Blender.Window.GetAreaSize()
		a=int(wid/3); x=int(hig/6)
		if x>30: x=30
		if a>180: a=180
		b=a/20;	c=a-(2*b) ;	y=x/10;	z=x-(2*y)
		#BGL.glClear(BGL.GL_COLOR_BUFFER_BIT)
		#BGL.glColor3f(1,1,1)
		#bottom row
		Draw.PushButton("<< Back",	 1, b, y, c, z,"Back to the main menu")
		Draw.PushButton("Reset",	 2, a+b,y,c,z,"Reset the values to the last saved") 
		Draw.PushButton("Apply",	 3, 2*a+b, y, c, z,"Apply the face settings")
		#other rows
		#Draw.PushButton(""										, 9,  c*3+b*2, 4*x+y, b*3, z,"Browse for directory in file selector")
		#Draw.PushButton(""										, 10, c*3+b*2, 3*x+y, b*3, z,"Browse for directory in file selector")
		#Draw.PushButton(""										, 11, c*3+b*2, 2*x+y, b*3, z,"Browse for directory in file selector")
		#Draw.PushButton(""										, 12, c*3+b*2, 1*x+y, b*3, z,"Browse for directory in file selector")
		
		self.togg1=Draw.String('BZFlag Texture Path: '		, 4, b, 4*x+y, c, z, self.bztexpath, 100, 'The directory on your machine where BZFlag stores standard textures')
		self.togg2=Draw.String('Custom Texture Path: '		, 5, b, 3*x+y, c, z, self.cstexpath, 100, 'The directory on your machine where you store custom textures')
		self.togg3=Draw.String('Cached Texture Path: '	, 6, b, 2*x+y, c, z, self.bzcachepath, 100, 'The directory on your machine where BZFlag stores custom textures')
		self.togg4=Draw.String('Custom Texture Host: '		, 7, b, 1*x+y, c, z, self.cstexhost, 100, 'The host you use to serve custom textures to other users')
		self.togg5=Draw.Slider('Rounding Accuracy: '				, 8, b, 5*x+y, c, z, self.ro , 0, 6, 1, "The number of decimal places the exporter will use when writing out files")
		#done
	

######################################################
#self.funtions()
######################################################
	def faceCleaner(self):
		pass
		Draw.Register(self.cleangui, self.cleanevent, self.cleanbutton_event)
		Draw.Redraw()
	#enddef


	def callback9(self,path):
		self.bztexpath=path
		Draw.Redraw()
	def callback10(self,path):
		self.cstexpath=path
		Draw.Redraw()
	def callback11(self,path):
		self.bzcachepath=path
		Draw.Redraw()

	def prefGet(self):
		BZPrefs = Blender.Registry.GetKey('BZFlag')
		self.bztexpath=BZPrefs['bztexpath']
		self.cstexpath=BZPrefs['cstexpath']
		self.cstexhost=BZPrefs['cstexhost']
		self.bzcachepath=BZPrefs['bzcachepath']
		self.ro=BZPrefs['ro']
		Draw.Redraw()
		return
		
	def prefSave(self):
		BZPrefs={}
		BZPrefs['bztexpath']=self.nbztexpath.val
		BZPrefs['cstexpath']=self.ncstexpath.val
		BZPrefs['cstexhost']=self.ncstexhost.val
		BZPrefs['bzcachepath']=self.nbzcachepath.val
		BZPrefs['ro']=self.nro.val
		if 'linux' in sys.platform:
			Blender.Registry.SetKey('BZFlag', BZPrefs, False)
			BZRegistry.SaveConfigData('BZFlag')
		else:
			Blender.Registry.SetKey('BZFlag', BZPrefs, True)
		
	
	
	def editPrefs(self):
		self.prefGet()
		
		
		
		
		Draw.Register(self.prefgui, self.prefevent, self.prefbutton_event)
		Draw.Redraw()		


	def addNew(self):
		Draw.Register(self.addgui, self.addevent, self.addbutton_event)
		Draw.Redraw()
	
	def editExisting(self):
		self.boolval={}#reset values
		self.bool={}
		self.stringval={}
		self.string={}
		self.materialval={}
		self.material={}
		self.attribval={}
		self.attrib={}#reset values
		obj=Blender.Object.Get(self.BZobjectlist[int(self.selectedBZObject.val)])
		BZType=self.propertyCheck(obj,'bzflag')
		if BZType:	self.template=BZtypeList.Objects[BZType]()
		else:		return
		if BZType in ['meshpyr','meshbox','arc','cone','sphere']:
			self.ExceptList=('spin','shear','scale','shift','name','position','size','rotation','ingroup')
		elif BZType in ['material']:
			self.ExceptList=('addtexture','texture','matref','name','ingroup')
		elif BZType =='mesh':
			self.ExceptList=('spin','shear','scale','shift','name','ingroup','position','size','rotation')
		else:
			self.ExceptList=('size','position','rotation','name','tilt','height','ingroup')
		for t in self.template.bools:
			if t not in self.ExceptList:	self.boolval[t]=self.propertyCheck(obj,t)
		for t in self.template.strings:
			if t not in self.ExceptList:	self.stringval[t]=self.propertyCheck(obj,t)
		for t in self.template.attribs:
			if t not in self.ExceptList:	self.attribval[t]=self.propertyCheck(obj,t)
		for t in self.template.materials:
			data=self.propertyCheck(obj,t)
			if data:				self.materialval[t]=self.BZmatIndexList[data]
			else:					self.materialval[t]=1
		self.template.name=obj.name
		Draw.Register(self.editgui, self.editevent, self.editbutton_event)
		Draw.Redraw()

	def applyExisting(self):
		obj=Blender.Object.Get(self.BZobjectlist[int(self.selectedBZObject.val)])
		self.template=BZtypeList.Objects[self.propertyCheck(obj,'bzflag')]()
		for t in self.template.bools:
			if t not in self.ExceptList:self.propertyCheck(obj,t,self.boolval[t])
		for t in self.template.strings:
			if t not in self.ExceptList:self.propertyCheck(obj,t,self.stringval[t])
		for t in self.template.attribs:
			if t not in self.ExceptList:self.propertyCheck(obj,t,self.attribval[t])
		for t in self.template.materials:
			data=self.BZmatobjectlist[self.materialval[t]]
			self.propertyCheck(obj,t,data)
		#reset base colours
		if self.template.type()=='base':
			BZPrefs=Blender.Registry.GetKey('BZFlag', True)
			colour=int(self.cleanStr(self.propertyCheck(obj,'color')))
			if colour in [1,2,3,4]:
				wallTextures=['','red_basewall.png','green_basewall.png','blue_basewall.png','purple_basewall.png']
				topTextures=['','red_basetop.png','green_basetop.png','blue_basetop.png','purple_basetop.png']
				colours=['','red','green','blue','purple']
				try:
					wallimage=Blender.Image.Get(wallTextures[colour])
				except NameError:
					wallimage=Blender.Image.Load(BZPrefs['bztexpath']+wallTextures[colour])
				try:
					topimage=Blender.Image.Get(topTextures[colour])
				except NameError:
					topimage=Blender.Image.Load(BZPrefs['bztexpath']+topTextures[colour])
				mesh=obj.getData()
				for i in xrange(2,6):
					mesh.faces[i].image=wallimage
				mesh.faces[1].image=topimage
				mesh.update()
		self.getobjects(Blender.Scene.GetCurrent())	
			

	def propertyCheck(self, object, name ,value="none"):	
		try:
			attrib=object.getProperty(name).getData()
			if value =="none":
				return attrib
			else:
				object.getProperty(name).setData(value)
				return
		except:
			if value =="none":
				return ''
			else:
				object.addProperty(name,value)
				return


	def getobjects(self, scene):
		try:
			material=Blender.Material.Get('BZtexface')
		except NameError:
			material=Blender.Material.New('BZtexface')
			material.setMode('TexFace')
			material.setSpec(0.1)
		objects = []
		objects = scene.getChildren()
		self.usedNames=[]
		self.menulist="Object: %t"
		self.BZmenulist="Object: %t"
		self.BZmatmenulist=": %t|None %x 1"
		self.BZmatobjectlist[1]='None'
		self.BZmatIndexList['None']=1
		self.baseMatMenu="%t|None %x 1"
		i=j=1
		k=2
		for obj in objects:
			self.usedNames.append(obj.name)
			if self.propertyCheck(obj,'bzflag'):
				self.BZmenulist=self.BZmenulist +"|%s " %(obj.name) + " %x" + str(j)
				self.BZobjectlist[j]=obj.name
				j+=1
				if self.propertyCheck(obj,'bzflag')=='material':
					self.BZmatmenulist=self.BZmatmenulist +"|%s " %(obj.name) + " %x" + str(k)
					self.BZmatobjectlist[k]=obj.name
					self.BZmatIndexList[obj.name]=k
					k+=1					
			elif obj.getType()=='Mesh' and not obj.getParent() and 1 in obj.layers:
				self.menulist=self.menulist +"|%s " %(obj.name) + " %x" + str(i)
				self.objectlist[i]=obj.name
				i+=1
		mat=Blender.Material.Get()
		Draw.Register(self.gui, self.event, self.button_event) #thunderbirds are go!!!!
		
			
	def checkName(self,name):
		newname=name
		a=1
		while newname in self.usedNames:
			newname='%s_%s' %(name,a)
			a+=1
		self.usedNames.append(newname)
		return newname
	
	
	
	
	def bitCheck(self,dec,bit):
		bitVars={}
		for i in range(0,20):	
			b=19-i
			if (dec-(2**b)) >= 0:
				bitVars[b]=1
				dec -=(2**b)
			else:
				bitVars[b]=0
		return bitVars[(bit)]
	def cleanStr(self,string):
		string=str(string)
		string=string.replace('\'','')
		string=string.replace('[','')
		string=string.replace(']','')
		string=string.replace(',','')
		return string
############# Main Program #############	

if Blender.Get('version') < 242:
	print "Warning: BZW Tool Box can't run in this Blender version!"
	print " You aren't running blender version >= 2.42"
	print " Upgrade your version from http://blender.org/"
	name = "Your Blender Verion is too old, you need 2.42 or higher.%t|Quit Blender to get a newer version?" 
	result = Blender.Draw.PupMenu(name)
	if result:
		res2=Blender.Draw.PupMenu('Really?%t|Yes|No')
		if res2==1: Blender.Quit()
	
		
else:
    #print "Startup"
    scene = Blender.Scene.GetCurrent()
    run=BZWTOOLKIT()
    run.getobjects(scene)
