# Copyright (C) 2006: A.D.Foster
#
# ***** BEGIN GPL LICENSE BLOCK *****
#
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# ***** END GPL LICENCE BLOCK *****
import BZtypeList
import BZcommon
import BZsceneWriter
import Blender
import sys
import BZRegistry
class writer:
	def __init__(self):
		# retrieve BZPrefs to avoid trying to load a file that isn't there
		if 'linux' in sys.platform:
			BZRegistry.LoadConfigData('BZFlag')
		# end mod retrieve
		self.debug=1
		self.groupList={}
		self.inGroup=False
		self.importedNames=[]
		self.index=0
		self.lineNo=0
		self.matDict={}
		self.matList=[]
		self.telelist=[]
		self.debug=1
		self.telelist.append(0)
	#enddef
	def write(self,filename):
		self.pipe=open(filename,'w')
		self.writeHeader(filename)
		self.writePrefix()
		self.writeWorld()
		self.exportDynCol()
		self.exportTexMat()
		self.exportPhyDrv()
		self.exportMaterial()
		self.makeMatDict()
		self.exportDefine()
		self.exportRest()
		self.exportGroups()
		self.exportLinks()
		self.writeSuffix()
		self.pipe.close()
		print '\n\n\t\033[1;32m **** Finished exporting scene to:-\033[0m%s ****' % filename

	def writeHeader(self,filename):
		if self.debug:print '\033[1;34mStarting export of Blener scene to:-\033[0m %s ' % filename
		self.pipe.write('####################################################\n########### BZW version 2.0.4 map file #############\n')
		self.pipe.flush()
		self.pipe.write('## Created by BZW-Import/Export Tools for Blender ##\n##    http://www.teppic.homelinux.net/BZWTools/   ##\n##           http://www.blender3d.org             ##\n####################################################\n')
		self.pipe.flush()

	def writeWorld(self):
		if self.debug:print '\033[1;34mStarting export of: world object\033[0m' 
		scene=Blender.Scene.GetCurrent()
		kids=scene.getChildren()
		for obj in kids:
			if obj.layers==[1] and BZcommon.propertyCheck(obj,'bzflag')=='world':
				module=BZsceneWriter.world
				writer=module.export(obj,self.pipe)
				writer.run()
			else:
				pass
		if self.debug:print '\033[1;32mFinished exporting world object\033[0m'

	def writePrefix(self):
		if self.debug:print '\033[1;34mStarting export of: options\033[0m' 
		for texts in Blender.Text.Get():
			if texts.getName() == "prefix":
				lines=texts.asLines()
				for x in xrange(texts.getNLines()):
					self.pipe.write("%s\n" % lines[x])
		self.pipe.flush()
		if self.debug:print '\033[1;32mFinished exporting options\033[0m'

	def exportDynCol(self):
		if self.debug:print '\033[1;34mStarting export of: dynamicColor objects\033[0m' 
		scene=Blender.Scene.GetCurrent()
		kids=scene.getChildren()
		for obj in kids:
			if obj.layers==[17]:
				module=BZsceneWriter.dyncol#()
				writer=module.export(obj,self.pipe)
				writer.run()
			else:
				pass
		if self.debug:print '\033[1;32mFinished exporting dynamicColors\033[0m'

	def exportTexMat(self):
		if self.debug:print '\033[1;34mStarting export of: textureMatrix objects\033[0m' 
		scene=Blender.Scene.GetCurrent()
		kids=scene.getChildren()
		for obj in kids:
			if obj.layers==[16]:
				module=BZsceneWriter.texmat
				writer=module.export(obj,self.pipe)
				writer.run()
			else:
				pass
		if self.debug:print '\033[1;32mFinished exporting textureMatrices\033[0m'

	def exportPhyDrv(self):
		if self.debug:print '\033[1;34mStarting export of: physics objects\033[0m' 
		scene=Blender.Scene.GetCurrent()
		kids=scene.getChildren()
		for obj in kids:
			if obj.layers==[18]:
				module=BZsceneWriter.physics
				writer=module.export(obj,self.pipe)
				writer.run()
			else:
				pass
		if self.debug:print '\033[1;32mFinished exporting physics\033[0m'

	def exportMaterial(self):
		if self.debug:print '\033[1;34mStarting export of: materials\033[0m' 
		scene=Blender.Scene.GetCurrent()
		kids=scene.getChildren()
		for obj in kids:
			if obj.layers==[20] and BZcommon.propertyCheck(obj,'bzflag')=='material':
				module=BZsceneWriter.material
				writer=module.export(obj,self.pipe)
				writer.run()
				mat=obj.getData().getMaterials()[0].name
				self.matList.append(mat)
			else:
				pass
		if self.debug:print '\033[1;32mFinished exporting materials\033[0m'

	def exportDefine(self):
		if self.debug:print '\033[1;34mStarting export of: group definitions\033[0m' 
		scene=Blender.Scene.GetCurrent()
		kids=scene.getChildren()
		for obj in kids:
			if obj.layers==[3] and obj.getType()=='Empty':
				module=BZsceneWriter.define
				writer=module.export(obj,self.pipe)
				writer.run()
			else:
				pass
		if self.debug:print '\033[1;32mFinished exporting group definitions\033[0m'

	def exportGroups(self):
		if self.debug:print '\033[1;34mStarting export of: group instances\033[0m' 
		scene=Blender.Scene.GetCurrent()
		kids=scene.getChildren()
		for obj in kids:
			if obj.layers==[2] and obj.getType()=='Empty':
				module=BZsceneWriter.group
				writer=module.export(obj,self.pipe)
				writer.run()
			else:
				pass
		if self.debug:print '\033[1;32mFinished exporting group instances\033[0m'
	
	def exportRest(self):
		if self.debug:print '\033[1;34mStarting export of: obstacle objects\033[0m' 
		scene=Blender.Scene.GetCurrent()
		kids=scene.getChildren()
		for obj in kids:
			if obj.layers==[1] and cleanStr(BZcommon.propertyCheck(obj,'bzflag')) not in ['world','group']:
				keywd=cleanStr(BZcommon.propertyCheck(obj,'bzflag'))
				if BZsceneWriter.Object.has_key(keywd): 
					module=BZsceneWriter.Object[keywd]
					writer=module.export(obj,self.pipe)
					writer.run()
			else:
				pass
		if self.debug:print '\033[1;32mFinished exporting obstacles\033[0m'

	def exportLinks(self):
		if self.debug:print '\033[1;34mStarting export of: links objects\033[0m' 
		scene=Blender.Scene.GetCurrent()
		kids=scene.getChildren()
		for obj in kids:
			if obj.layers==[19]:
				module=BZsceneWriter.link
				writer=module.export(obj,self.pipe)
				writer.run()
			else:
				pass
		if self.debug:print '\033[1;32mFinished exporting links\033[0m'

	def writeSuffix(self):
		if self.debug:print '\033[1;34mStarting export of: post export text\033[0m' 
		for texts in Blender.Text.Get():
			if texts.getName() == "suffix":
				lines=texts.asLines()
				for x in xrange(texts.getNLines()):
					self.pipe.write("%s\n" % lines[x])
		self.pipe.flush()
		if self.debug:print '\033[1;32mFinished exporting text\033[0m'

	def makeMatDict(self):
		mats=Blender.Material.Get()
		for mat in mats:
			image=self.getImage(mat.name)
			if image:
				if mat.name in self.matList:
					self.matDict[image.name]=mat.name
				else:
					print 'Material %s has no BZMaterial properties, and the texture will not be exported' % mat.name

	def getImage(self,matref):
		mat=Blender.Material.Get(matref)
		try:
			tex=mat.getTextures()[0].tex
			img=tex.getImage()
		except AttributeError:
			#print 'Texture not found %s' % matref
			img=None
		return img
	#enddef


def cleanStr(string):
	string=str(string)
	string=string.replace('\'','')
	string=string.replace('[','')
	string=string.replace(']','')
	string=string.replace(',','')
	return string


def retList(line):
	retlist=[]
	line=line.replace('\t',' ')
	#line=line.replace(',',' ')  ##not needed.
	while line.find('  ')>=0:
		line=line.replace('  ',' ')
	line=line.strip()
	line=line + ' '
	while line!='':
		retlist.append(line[:line.find(' ')])
		line=line[line.find(' ')+1:]
	return retlist
#enddef
def fatalError():
	print '\033[1;31m#####################################################'
	print '#######  Exited on fatal map file error!  ###########'
	print '#####################################################\033[0m'
	sys.exit()
#enddef
